import { css, LitElement, property, svg } from "lit-element";

const NODE_SIZE = 24;
const VERTICAL_SPACING = 10;

export class HatGraphNode extends LitElement {
  @property() iconPath?;
  @property({ reflect: true }) marked?;
  @property({ reflect: true }) selected?;
  @property({ reflect: true }) disabled?;

  connectedCallback() {
    super.connectedCallback();
    if (!this.hasAttribute("tabindex")) this.setAttribute("tabindex", "0");
    this.addEventListener("focusin", () => this.setAttribute("selected", ""));
    this.addEventListener("focusout", () => this.removeAttribute("selected"));
  }

  updated() {
    const svg = this.shadowRoot.querySelector("svg");
    const bbox = svg.getBBox();
    svg.setAttribute("width", `${bbox.width + 2}px`);
    svg.setAttribute("height", `${bbox.height + 1}px`);
    svg.setAttribute(
      "viewBox",
      `${bbox.x - 1} ${bbox.y} ${bbox.width + 2} ${bbox.height + 1}`
    );
  }

  render() {
    return svg`
    <svg
    >
      <path
        class="connector"
        d="
          M 0 ${-VERTICAL_SPACING}
          L 0 0
        "
        line-caps="round"
      />
    <g class="node"
      transform="translate(0 ${VERTICAL_SPACING})"
      >
      <circle
        cx="0"
        cy="0"
        r="${NODE_SIZE / 2}"
      />
      <g
        style="pointer-events: none"
        transform="translate(${-12} ${-12})"
      >
        ${this.iconPath ? svg`<path d="${this.iconPath}"/>` : ""}
      </g>
    </g>
      </svg>
    `;
  }

  static get styles() {
    return css`
      :host {
        display: flex;
        flex-direction: column;
        --stroke-clr: var(--stroke-color, rgb(3, 169, 244));
        --selected-clr: var(--selected-color, rgb(255, 152, 0));
        --marked-clr: var(--marked-color, green);
        --hover-clr: var(--hover-color, red);
        --disabled-clr: var(--disabled-color, gray);
      }
      :host([selected]),
      :host(:focus) {
        --circle-clr: var(--selected-clr);
        outline: none;
      }
      :host([marked]) {
        --stroke-clr: var(--marked-clr);
      }
      :host(:hover) circle {
        --stroke-clr: var(--hover-clr);
      }
      :host([disabled]) circle {
        stroke: var(--disabled-clr);
      }
      :host-context([disabled]) {
        --stroke-clr: var(--disabled-clr);
      }

      circle,
      path.connector {
        stroke: var(--stroke-clr);
        stroke-width: 2;
        fill: none;
      }
      circle {
        fill: white;
        stroke: var(--circle-clr, var(--stroke-clr));
      }
    `;
  }
}

customElements.define("hat-graph-node", HatGraphNode);
